##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
XML DOM Base classes
"""

from Node import Element, Text, Document, ProcessingInstruction

class ParseError(Exception): pass

try:
    from Shared.DC.xml import pyexpat
except ImportError:
    try:
        import pyexpat
    except ImportError:
        from xml.parsers import pyexpat

class Parser:

    buffer_size=1028 * 64

    def __init__(self):
        self.Element=Element
        self.Text=Text
        self.Document=Document
        self.ProcessingInstruction=ProcessingInstruction
        self.reset()
        
    def reset(self, doc=None):
        if doc is None:
            doc=self.Document("#document")
        self.cur_elem = self.root = doc
        self.error = None

    def register(self, node, klass):
        # register a class for a given node type
        setattr(self, node, klass)

    def start(self, name, attrs):
        e=self.Element(name)      
        if self.cur_elem is not None:
            self.cur_elem._append_child(e)
            self.cur_elem = e.__of__(self.cur_elem)
        else:
            self.cur_elem = self.root = e
        for i in range(0, len(attrs) ,2):
            self.cur_elem.setAttribute(attrs[i], attrs[i+1])
             
    def end(self, name):    
        try: self.cur_elem=self.cur_elem.aq_parent      
        except: pass
            
    def cdata(self, data):
        e=self.Text(data)
        self.cur_elem._append_child(e)
    
    def pi(self, target, data):
        e=self.ProcessingInstruction(target, data)
        self.cur_elem._append_child(e)
        
    def parse(self, input):
        p = pyexpat.ParserCreate()
        p.StartElementHandler = self.start
        p.EndElementHandler = self.end
        p.CharacterDataHandler = self.cdata
        p.ProcessingInstructionHandler = self.pi
        
        if type(input)==type(''):
            rv = p.Parse(input, 1)
        else:
            while 1:
                v=input.read(self.buffer_size)
                if v == "":
                    break
                rv = p.Parse(v, 0)
                if not rv:
                    break
        if not rv:
            raise ParseError(pyexpat.ErrorString(p.ErrorCode))
        else:
            self.root.normalize()
            return self.root

if __name__=="__main__":
    p=Parser()
    r=p.parse("""<?xml version="1.0"?>
<one blah="bort" fort="tort">
<single/>
This is some text
<tee> &amp; <inner/> </tee>
<two> blah </two>
<two> bort </two>
<three></three>
</one>
""")

    print r.toXML()
    




